﻿using System;

namespace JsonRpcLite.Services
{
    /// <summary>
    /// Attribute for the service.
    /// </summary>
    public class RpcServiceAttribute : Attribute
    {
        /// <summary>
        /// Gets the Url string
        /// </summary>
        public string Name { get; }

        /// <summary>
        /// Enable the smd service if it is true, otherwise is false
        /// </summary>
        public bool EnableSmd { get; }

        /// <summary>
        /// Create attribute for given name.
        /// </summary>
        /// <param name="name">The service name</param>
        /// <param name="enableSmd">Enable the smd service if it is true, otherwise is false</param>
        public RpcServiceAttribute(string name, bool enableSmd)
        {
            Name = name;
            EnableSmd = enableSmd;
        }

        /// <summary>
        /// Create attribute for given name.
        /// </summary>
        /// <param name="name">The service name</param>
        public RpcServiceAttribute(string name)
        {
            Name = name;
            EnableSmd = false;
        }

        /// <summary>
        /// Create attribute for given name.
        /// </summary>
        /// <param name="enableSmd">Enable the smd service if it is true, otherwise is false</param>
        public RpcServiceAttribute(bool enableSmd)
        {
            Name = string.Empty;
            EnableSmd = enableSmd;
        }


        /// <summary>
        /// Create attribute for given name.
        /// </summary>
        public RpcServiceAttribute()
        {
            Name = string.Empty;
            EnableSmd = false;
        }
    }


    /// <summary>
    /// The JsonRpcMethod Attribute, means this method is a JsonRpc method
    /// </summary>
    public class RpcMethodAttribute : Attribute
    {
        /// <summary>
        /// Gets the name of this method.
        /// </summary>
        public string Name { get; }

        public RpcMethodAttribute()
        {
            Name = string.Empty;
        }

        public RpcMethodAttribute(string name)
        {
            Name = name;
        }
    }
}
